@@ -34,7 +34,6 @@ import logging
 from authomatic import providers
 from authomatic.exceptions import CancellationError, FailureError, OAuth2Error
 import authomatic.core as core
-import authomatic.settings as settings
 
 __all__ = ['OAuth2', 'Behance', 'Bitly', 'Cosm', 'DeviantART', 'Facebook', 'Foursquare', 'GitHub',
            'Google', 'LinkedIn', 'PayPal', 'Reddit', 'Viadeo', 'VK', 'WindowsLive', 'Yammer', 'Yandex']
@@ -94,7 +93,7 @@ class OAuth2(providers.AuthorizationProvider):
     
     @classmethod
     def create_request_elements(cls, request_type, credentials, url, method='GET', params=None,
-                                headers=None, body='', redirect_uri='', scope='', csrf=''):
+                                headers=None, body='', secret=None, redirect_uri='', scope='', csrf=''):
         """
         Creates |oauth2| request elements.
         """
@@ -115,11 +114,12 @@ class OAuth2(providers.AuthorizationProvider):
         
         if request_type == cls.USER_AUTHORIZATION_REQUEST_TYPE:
             # User authorization request.
+            # TODO: Raise error for specific message for each missing argument.
             if consumer_key and redirect_uri and (csrf or not cls.supports_csrf_protection):
                 params['client_id'] = consumer_key
                 params['redirect_uri'] = redirect_uri
                 params['scope'] = scope
-                params['state'] = csrf or cls.csrf_generator()
+                params['state'] = csrf
                 params['response_type'] = 'code'
                 
                 # Add authorization header
@@ -137,11 +137,11 @@ class OAuth2(providers.AuthorizationProvider):
                 params['redirect_uri'] = redirect_uri
                 params['grant_type'] = 'authorization_code'
                 
-                # TODO: Chenck whether all providers accept it
+                # TODO: Check whether all providers accept it
                 headers.update(cls._authorization_header(credentials))
             else:
                 raise OAuth2Error('Credentials with valid token, consumer_key, consumer_secret and argument ' + \
-                                  'redirect_uri are required to create OAuth 2.0 acces token request elements!')
+                                  'redirect_uri are required to create OAuth 2.0 access token request elements!')
         
         elif request_type == cls.REFRESH_TOKEN_REQUEST_TYPE:
             # Refresh access token request.
@@ -217,8 +217,7 @@ class OAuth2(providers.AuthorizationProvider):
         return credentials
     
     
-    @classmethod
-    def refresh_credentials(cls, credentials):
+    def refresh_credentials(self, credentials):
         """
         Refreshes :class:`.Credentials` if it gives sense.
         
@@ -229,21 +228,21 @@ class OAuth2(providers.AuthorizationProvider):
             :class:`.Response`.
         """
         
-        if not cls._x_refresh_credentials_if(credentials):
+        if not self._x_refresh_credentials_if(credentials):
             return
         
         # We need consumer key and secret to make this kind of request.
-        cfg = settings.config.get(credentials.provider_name)
+        cfg = credentials.config.get(credentials.provider_name)
         credentials.consumer_key = cfg.get('consumer_key')
         credentials.consumer_secret = cfg.get('consumer_secret')
         
-        request_elements = cls.create_request_elements(request_type=cls.REFRESH_TOKEN_REQUEST_TYPE,
+        request_elements = self.create_request_elements(request_type=self.REFRESH_TOKEN_REQUEST_TYPE,
                                                         credentials=credentials,
-                                                        url=cls.access_token_url,
+                                                        url=self.access_token_url,
                                                         method='POST')
         
-        cls._log(logging.INFO, 'Refreshing credentials.')
-        response = cls._fetch(*request_elements)
+        self._log(logging.INFO, 'Refreshing credentials.')
+        response = self._fetch(*request_elements)
         
         # We no longer need consumer info.
         credentials.consumer_key = None
@@ -263,7 +262,7 @@ class OAuth2(providers.AuthorizationProvider):
                 credentials.refresh_token = refresh_token
             
             # Handle different naming conventions across providers.
-            credentials = cls._x_credentials_parser(credentials, response.data)
+            credentials = self._x_credentials_parser(credentials, response.data)
         
         return response
     
@@ -380,7 +379,7 @@ class OAuth2(providers.AuthorizationProvider):
             csrf = ''
             if self.supports_csrf_protection:
                 # generate csfr
-                csrf = self.csrf_generator()
+                csrf = self.csrf_generator(self.settings.secret)
                 # and store it to session
                 self._session_set('state', csrf)
             else:
@@ -550,7 +549,7 @@ class Facebook(OAuth2):
     def _x_request_elements_filter(cls, request_type, request_elements, credentials):
         
         if request_type == cls.REFRESH_TOKEN_REQUEST_TYPE:
-            # As allways, Facebook has it's original name for "refresh_token"!
+            # As always, Facebook has it's original name for "refresh_token"!
             url, method, params, headers, body = request_elements
             params['fb_exchange_token'] = params.pop('refresh_token')
             params['grant_type'] = 'fb_exchange_token'
@@ -592,7 +591,7 @@ class Facebook(OAuth2):
     
     @staticmethod
     def _x_refresh_credentials_if(credentials):
-        # Allways refresh.
+        # Always refresh.
         return True
 
 
@@ -652,6 +651,27 @@ class GitHub(OAuth2):
     * Dashboard: https://github.com/settings/applications/
     * Docs: http://developer.github.com/v3/#authentication
     * API reference: http://developer.github.com/v3/
+    
+    .. note::
+        
+        GitHub API `documentation <http://developer.github.com/v3/#user-agent-required>`_ sais:
+        
+            all API requests MUST include a valid ``User-Agent`` header.
+        
+        You can apply a default ``User-Agent`` header for all API calls in the config like this:
+        
+        .. code-block:: python
+            :emphasize-lines: 6
+        
+            CONFIG = {
+                'github': {
+                    'class_': oauth2.GitHub,
+                    'consumer_key': '#####',
+                    'consumer_secret': '#####',
+                    'access_headers': {'User-Agent': 'Awesome-Octocat-App'},
+                }
+            }
+    
     """
     
     user_authorization_url = 'https://github.com/login/oauth/authorize'
@@ -665,7 +685,14 @@ class GitHub(OAuth2):
         user.username = data.get('login')
         user.picture = data.get('avatar_url')
         user.link = data.get('html_url')
-        user.city, user.country = data.get('location', ', ').split(', ')
+        
+        user.city = data.get('location', '')
+        if ',' in user.city:
+            location = user.city.split(',')
+            user.city = location[0].strip()
+            if len(location) > 1:
+                user.country = location[1].strip()
+            
         return user
     
     @classmethod
@@ -687,7 +714,7 @@ class Google(OAuth2):
     
     user_authorization_url = 'https://accounts.google.com/o/oauth2/auth'
     access_token_url = 'https://accounts.google.com/o/oauth2/token'
-    user_info_url = 'https://www.googleapis.com/oauth2/v1/userinfo'
+    user_info_url = 'https://www.googleapis.com/oauth2/v3/userinfo'
     
     user_info_scope = ['https://www.googleapis.com/auth/userinfo.profile',
                        'https://www.googleapis.com/auth/userinfo.email']
@@ -707,9 +734,15 @@ class Google(OAuth2):
     
     @staticmethod
     def _x_user_parser(user, data):
+        user.id = data.get('sub') or data.get('id')
         user.name = data.get('name')
         user.first_name = data.get('given_name')
         user.last_name = data.get('family_name')
+        user.link = data.get('profile')
+        try:
+            user.birth_date = datetime.datetime.strptime(data.get('birthdate'), "%Y-%m-%d")
+        except:
+            user.birth_date = data.get('birthdate')
         return user
     
     
@@ -905,7 +938,7 @@ class VK(OAuth2):
                 'consumer_key': '#####',
                 'consumer_secret': '#####',
                 'id': authomatic.provider_id(),
-                'scope': ['1024'] # Allways a single item.
+                'scope': ['1024'] # Always a single item.
             }
         }
     
@@ -1074,7 +1107,7 @@ class Yandex(OAuth2):
 
 
 # The provider type ID is generated from this list's indexes!
-# Allways apppend new providers at the end so that ids of existing providers don't change!
+# Always append new providers at the end so that ids of existing providers don't change!
 PROVIDER_ID_MAP = [OAuth2, Behance, Bitly, Cosm, DeviantART, Facebook, Foursquare, GitHub, Google, LinkedIn,
           PayPal, Reddit, Viadeo, VK, WindowsLive, Yammer, Yandex]
 
